# -*- coding: utf-8 -*-
import os,sys,bz2
import unittest
from six.moves import cPickle as pickle

import shutil
import tempfile
import pykakasi.genkanwadict as genkanwadict

class TestGenkanwadict(unittest.TestCase):
    kanwa = None
    def setUp(self):
        self.tmpdir = tempfile.mkdtemp()

    def tearDown(self):
        shutil.rmtree(self.tmpdir, ignore_errors=True)

    def constructor(self):
        self.kanwa = genkanwadict.mkkanwa()
        self.assertEqual(self.kanwa, object)

    def test_mkdict(self):
        if self.kanwa is None:
            self.kanwa = genkanwadict.mkkanwa()

        src = os.path.join('tests','kanadict.utf8')
        dst = os.path.join(self.tmpdir,'test_kanadict.pickle')
        self.kanwa.mkdict(src, dst)
        # load test
        with open(dst,'rb') as f:
            (mydict, maxkeylen) = pickle.load(f)
            f.close()
        self.assertTrue(isinstance(mydict, dict))
        self.assertEqual(maxkeylen, 3)

    def test_mkkanwa(self):
        if self.kanwa is None:
            self.kanwa = genkanwadict.mkkanwa()

        src = os.path.join('tests','kakasidict.utf8')
        dst = os.path.join(self.tmpdir,'test_kanwadict3.db') # FIXME:  no hardcoded filename
        self.kanwa.run(src, dst)



# -*- coding: utf-8 -*-
#  kanwa.py
#
# Copyright 2011-2018 Hiroshi Miura <miurahr@linux.com>
from zlib import decompress
from pkg_resources import resource_filename
from marshal import loads
import six
import semidbm as dbm

class kanwa (object):

# This class is Borg/Singleton
# It provides same results becase lookup from a static dictionary.
# There is no state rather dictionary dbm.
    _shared_state = {}

    _kanwadict = None
    _jisyo_table = {}

# FIXME: there is no invalidate mechanism for _jisyo_table data.
#        It can lead a large memory consumption in long live process.
#        but max size are limited to a size of a dictionary.

    def __new__(cls, *p, **k):
        self = object.__new__(cls, *p, **k)
        self.__dict__ = cls._shared_state
        return self

    def __init__(self):
        if self._kanwadict is None:
            dictpath = resource_filename(__name__, 'kanwadict3.db') # FIXME: no hardcoded filename
            self._kanwadict = dbm.open(dictpath,'r')

    def load(self, char):
        if six.PY2:
            key = "%04x"%ord(unicode(char))
        else:
            key = "%04x"%ord(char)
        if key in self._jisyo_table:
            return self._jisyo_table[key]
        else:
            try:
                self._jisyo_table[key]  = loads(decompress(self._kanwadict[key]))
                return self._jisyo_table[key]
            except:
                return None




from .a2 import a2
from .j2h import J2H
from .j2k import J2K
from .k2a import K2a
from .h2a import H2a
from .h2k import H2K
from .k2h import K2H
from .j2a import J2a
from .symbols import sym2
from .kakasi import kakasi
from .wakati import wakati
from .exceptions import *

__all__ = ["pykakasi"]




# -*- coding: utf-8 -*-
#  wakati.py
#
# Copyright 2014 Hiroshi Miura <miurahr@linux.com>
#
# * This program is free software; you can redistribute it and/or modify
# * it under the terms of the GNU General Public License as published by
# * the Free Software Foundation; either versions 2, or (at your option)
# * any later version.
# *
# * This program is distributed in the hope that it will be useful
# * but WITHOUT ANY WARRANTY; without even the implied warranty of
# * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# * GNU General Public License for more details.
# *
# * You should have received a copy of the GNU General Public License
# * along with KAKASI, see the file COPYING.  If not, write to the Free
# * Software Foundation Inc., 59 Temple Place - Suite 330, Boston, MA
# * 02111-1307, USA.
# */

import re
import sys, os
from .kakasi import kakasi

class wakati(kakasi):

    _jconv = None
    _separator = " "
    _state = True

    def __init__(self):
        from .j2h import J2H
        self._jconv = J2H()

    def getConverter(self):
        return self

    def do(self, text):

        if len(text) == 0:
            return ""

        otext =  ''
        i = 0
        while True:
            if i >= len(text):
                break

            if self._jconv.isRegion(text[i]):
                _, l = self._jconv.convert(text[i:])
                if l <= 0:
                    otext = otext + text[i]
                    i += 1
                    self._state = False
                elif (i+l) < len(text):
                    if self._state:
                        otext = otext + text[i:i+l] + self._separator
                    else:
                        otext = otext + self._separator + text[i:i+l] + self._separator
                        self._state = True
                    i = i + l
                else:
                    if self._state:
                        otext = otext + text[i:i+l]
                    else:
                        otext = otext + self._separator + text[i:i+l]
                    break

            else:
                self._state = False
                otext  = otext + text[i]
                i += 1

        return otext




#!/usr/bin/env python
from zlib import compress
import re
from marshal import dumps
from six.moves.cPickle import dump
import six
import semidbm as dbm

class mkkanwa(object):

    records = {}

    def run(self, src, dst):
        with open(src, 'rb') as f:
            for line in f:
                self.parsekdict(line.decode("utf-8"))
            f.close()
        self.kanwaout(dst)

# for itaiji and kana dict

    def mkdict(self, src, dst):
        max_key_len = 0
        dic = {}
        with open(src, "rb") as f:
            for line in f:
                line = line.decode("utf-8").strip()
                if line.startswith(';;'): # skip comment
                    continue
                if re.match(r"^$",line):
                    continue
                try:
                    (v, k) = (re.sub(r'\\u([0-9a-fA-F]{4})',
                            lambda x:six.unichr(int(x.group(1),16)), line)).split(' ')
                    dic[k] = v
                    max_key_len = max(max_key_len, len(k))
                except:
                    raise Exception("Cannot process dictionary line: ", line)
        with open(dst, 'wb') as d:
            dump((dic, max_key_len), d, protocol=2)

# for kanwadict

    def parsekdict(self, line):
        line = line.strip()
        if line.startswith(';;'): # skip comment
            return
        (yomi, kanji) = line.split(' ')
        if ord(yomi[-1:]) <= ord('z'): 
            tail = yomi[-1:]
            yomi = yomi[:-1]
        else:
            tail = ''
        self.updaterec(kanji, yomi, tail)

    def updaterec(self, kanji, yomi, tail):
            key = "%04x"%ord(kanji[0]) 
            if key in self.records:
                if kanji in self.records[key]:
                    rec = self.records[key][kanji]
                    rec.append((yomi,tail))
                    self.records[key].update( {kanji: rec} )
                else:
                    self.records[key][kanji]=[(yomi, tail)]
            else:
                self.records[key] = {}
                self.records[key][kanji]=[(yomi, tail)]

    def kanwaout(self, out):
        dic = dbm.open(out, 'c')
        for (k, v) in self.records.items():
            dic[k] = compress(dumps(v))
        dic.close()



#!/usr/bin/env python
# derivered from unihandecode setup.py

import os, sys
from setuptools import Command,setup
from setuptools.command.install import install
from distutils.command.build import build

import pykakasi.genkanwadict as genkanwadict

def gen_dict(src_f, pkl_f):
    kanwa = genkanwadict.mkkanwa()
    src = os.path.join('pykakasi', 'data', src_f)
    dst = os.path.join('pykakasi', pkl_f)
    try:
        os.unlink(dst)
    except:
        pass
    kanwa.mkdict(src, dst)

def gen_kanwa(src, dst):
    try:
        os.unlink(dst+'.db')
    except:
        pass
    kanwa = genkanwadict.mkkanwa()
    kanwa.run(src, dst)

def readme():
    with open(os.path.join(os.path.dirname(__file__),'README.rst')) as f:
        return f.read()

def pre_build():
    DICTS = [
        ('itaijidict.utf8', 'itaijidict2.pickle'),
        ('hepburndict.utf8', 'hepburndict2.pickle'),
        ('kunreidict.utf8', 'kunreidict2.pickle'),
        ('passportdict.utf8', 'passportdict2.pickle'),
        ('hepburnhira.utf8', 'hepburnhira2.pickle'),
        ('kunreihira.utf8', 'kunreihira2.pickle'),
        ('passporthira.utf8', 'passporthira2.pickle')
    ]
    for (s,p) in DICTS:
        gen_dict(s, p)

    # build kakasi dict
    src = os.path.join('pykakasi','data','kakasidict.utf8')
    dst = os.path.join('pykakasi','kanwadict3.db')
    gen_kanwa(src, dst)

class my_build(build):
    def run(self):
        self.execute(pre_build, (),
                    msg="Running pre build task")
        build.run(self)

tests_require = ['nose','coverage','mock']
if sys.version_info < (2, 7):
    tests_require.append('unittest2')

setup(name='pykakasi',
      version='0.80',
      description='Python implementation of kakasi - kana kanji simple inversion library',
      url='http://github.com/miurahr/pykakasi',
      license='GPLv3',
      long_description=readme(),
      author='Hioshi Miura',
      author_email='miurahr@linux.com',
      packages = [ 'pykakasi',
                   'pykakasi.genkanwadict'
                 ],
      provides = [ 'pykakasi' ],
      scripts = ["kakasi"],
      include_package_data = True,
      package_data = {'pykakasi':  ['*.pickle'],
                      'pykakasi/kanwadict3.db': ['data']},
      test_suite = 'nose.collector',
      tests_require = tests_require,
      install_requires=['six','semidbm'],
      cmdclass = {
          'build':my_build
        }
)



# -*- coding: utf-8 -*-
#  exceptions.py
#
# Copyright 2015 Hiroshi Miura <miurahr@linux.com>
#

class PyKakasiException(Exception):
    pass

class UnknownCharacterException(PyKakasiException):
    pass

class UnsupportedRomanRulesException(PyKakasiException):
    pass

class UnknownOptionsException(PyKakasiException):
    pass

class InvalidModeValueException(PyKakasiException):
    pass

class InvalidFlagValueException(PyKakasiException):
    pass



# -*- coding: utf-8 -*-
import unittest
import pykakasi

class TestPyKakasi(unittest.TestCase):

    def test_kakasi_hepburn(self):

        TESTS = [
            (u"",""),
            (u"構成",         "Kousei"),
            (u"好き",         "Suki"),
            (u"大きい",       "Ookii"),
            (u"かんたん",     "kantan"),
            (u"にゃ",         "nya"),
            (u"っき",         "kki"),
            (u"っふぁ",       "ffa"),
            (u"漢字とひらがな交じり文", "Kanji tohiragana Majiri Bun"),
            (u"Alphabet 123 and 漢字", "Alphabet 123 and Kanji"),
            (u"日経新聞", "Nikkeishinbun"),
            (u"日本国民は、","Nihonkokumin ha,")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Hepburn")
        kakasi.setMode("s", True)
        kakasi.setMode("E", "a")
        kakasi.setMode("a", None)
        kakasi.setMode("C", True)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_kunrei(self):

        TESTS = [
            (u"", ""),
            (u"構成",         "Kousei"),
            (u"好き",          "Suki"),
            (u"大きい",       "Ookii"),
            (u"かんたん",     "kantan"),
            (u"にゃ",          "nya"),
            (u"っき",           "kki"),
            (u"っふぁ",        "ffa"),
            (u"漢字とひらがな交じり文", "Kanzi tohiragana Maziri Bun"),
            (u"Alphabet 123 and 漢字", "Alphabet 123 and Kanzi"),
            (u"日経新聞",     "Nikkeisinbun"),
            (u"日本国民は、", "Nihonkokumin ha,")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Kunrei")
        kakasi.setMode("E","a")
        kakasi.setMode("s", True)
        kakasi.setMode("a", None)
        kakasi.setMode("C", True)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_J2H(self):

        TESTS = [
            (u"",""),
            (u"構成",         u"こうせい"),
            (u"好き",         u"すき"),
            (u"大きい",       u"おおきい"),
            (u"かんたん",     u"かんたん"),
            (u"にゃ",         u"にゃ"),
            (u"っき",         u"っき"),
            (u"っふぁ",       u"っふぁ"),
            (u"漢字とひらがな交じり文", u"かんじとひらがなまじりぶん"),
            (u"Alphabet 123 and 漢字", u"Alphabet 123 and かんじ"),
            (u"日経新聞",     u"にっけいしんぶん"),
            (u"日本国民は、", u"にほんこくみんは、")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H",None)
        kakasi.setMode("K",None)
        kakasi.setMode("J","H")
        kakasi.setMode("s",False)
        kakasi.setMode("C",True)
        kakasi.setMode("E",None)
        kakasi.setMode("a",None)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_J2K(self):

        TESTS = [
            (u"",""),
            (u"構成",         u"コウセイ"),
            (u"好き",         u"スキ"),
            (u"大きい",       u"オオキイ"),
            (u"かんたん",     u"かんたん"),
            (u"漢字とひらがな交じり文", u"カンジとひらがなマジリブン"),
            (u"Alphabet 123 and 漢字", u"Alphabet 123 and カンジ")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H",None)
        kakasi.setMode("K",None)
        kakasi.setMode("J","K")
        kakasi.setMode("s",False)
        kakasi.setMode("C",True)
        kakasi.setMode("E",None)
        kakasi.setMode("a",None)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)


    def test_kakasi_H2K(self):

        TESTS = [
            (u"",""),
            (u"かんたん",     u"カンタン"),
            (u"にゃ",         u"ニャ")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","K")
        kakasi.setMode("S"," ")
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_K2H(self):

        TESTS = [
            (u"",""),
            (u"カンタン",u"かんたん"),
            (u"ニャ",    u"にゃ")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("K","H")
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)
 
    def test_wakati(self):
        TESTS = [
                (u"",u""),
                (u"交じり文", u"交じり 文"),
                (u"ひらがな交じり文", u"ひらがな 交じり 文"),
                (u"漢字とひらがな交じり文", u"漢字 とひらがな 交じり 文")
        ]
        wakati = pykakasi.wakati()
        converter = wakati.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_a2E(self):

        TESTS = [
            ("ABCDEFGHIJKLMNOPQRSTUVWXYZ",
             u"ＡＢＣＤＥＦＧＨＩＪＫＬＭＮＯＰＱＲＳＴＵＶＷＸＹＺ"),
            ("abcdefghijklmnopqrstuvwxyz",
             u"ａｂｃｄｅｆｇｈｉｊｋｌｍｎｏｐｑｒｓｔｕｖｗｘｙｚ"),
            ("!\"#$%&'()*+,-./_ {|}~",
             u"！＂＃＄％＆＇（）＊＋，－．／＿　｛｜｝～")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("a","E")
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_passport(self):

        TESTS = [
            (u"",""),
            (u"構成",         "Kosei"),
            (u"大野",         "Ono"),
            (u"斎藤",         "Saito"),
            (u"菅野",         "Kanno"),
            (u"本田",         "Honda"),
            (u"一式",         "Isshiki"),
            (u"別府",         "Beppu"),
            (u"ジェ",         "jie"),
            (u"チェ",         "chie"),
            (u"ティ",         "tei"),
            (u"ディ",         "dei"),
            (u"デュ",         "deyu"),
            (u"ファ",         "fua"),
            (u"フィ",         "fui"),
            (u"フェ",         "fue"),
            (u"フォ",         "fuo"),
            (u"ヴァ",         "bua"),
            (u"ヴィ",         "bui"),
            (u"ヴ",           "bu"),
            (u"ヴェ",         "bue"),
            (u"ヴォ",         "buo"),
            (u"じぇ",         "jie"),
            (u"ちぇ",         "chie"),
            (u"てぃ",         "tei"),
            (u"でぃ",         "dei"),
            (u"でゅ",         "deyu"),
            (u"ふぁ",         "fua"),
            (u"ふぃ",         "fui"),
            (u"ふぇ",         "fue"),
            (u"ふぉ",         "fuo")
         ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Passport")
        kakasi.setMode("E","a")
        kakasi.setMode("C",True)
        kakasi.setMode("a",None)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)


    @unittest.skip("Not yet implemented properly")
    def test_kakasi_passport_specialcase(self):

        TESTS = [
            (u"えっちゅう",   "etchu"),
            (u"はっちょう",   "hatcho"),
            (u"エッチュウ",   "etchu"),
            (u"ハッチョウ",   "hatcho")
         ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Passport")
        kakasi.setMode("E","a")
        kakasi.setMode("a",None)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)

    def test_kakasi_hepburn_nocapital(self):

        TESTS = [
            (u"",""),
            (u"構成",         "kousei"),
            (u"好き",         "suki"),
            (u"大きい",       "ookii"),
            (u"かんたん",     "kantan"),
            (u"にゃ",         "nya"),
            (u"っき",         "kki"),
            (u"っふぁ",       "ffa"),
            (u"漢字とひらがな交じり文", "kanji tohiragana majiri bun"),
            (u"Alphabet 123 and 漢字", "Alphabet 123 and kanji"),
            (u"日経新聞", "nikkeishinbun"),
            (u"日本国民は、","nihonkokumin ha,")
        ]

        kakasi = pykakasi.kakasi()
        kakasi.setMode("H","a")
        kakasi.setMode("K","a")
        kakasi.setMode("J","a")
        kakasi.setMode("r","Hepburn")
        kakasi.setMode("s", True)
        kakasi.setMode("E","a")
        kakasi.setMode("a", None)
        kakasi.setMode("C", False)
        converter  = kakasi.getConverter()
        for case, result in TESTS:
            self.assertEqual(converter.do(case), result)
